==> next-ft-api-client-3.6.0.tgz: package/src/related-tags.js <==
'use strict';

var fetchres = require('fetchres');
var logger = require('ft-next-logger');
var eagerFetch = require('./utils/eager-fetch');

function formatSearchTerm(options) {
	return Object.defineProperty({}, 'item.metadata.' + options.taxonomy + '.term.id', {
		value: options.id,
		enumerable: true
	});
}

function formatDateRange(from, to) {
	var dateFrom = new Date();
	var dateTo = new Date();

	dateFrom.setDate(dateFrom.getDate() - from);
	dateTo.setDate(dateTo.getDate() - to);

	return {
		gte: dateFrom.toISOString(),
		lt: dateTo.toISOString()
	};
}

function formatAggregations(options) {
	return options.taxonomies.reduce(function(previous, taxonomy) {
		return Object.defineProperty(previous, taxonomy, {
			enumerable: true,
			value: {
				terms: {
					field: 'item.metadata.' + taxonomy + '.term.id',
					exclude: options.id,
					min_doc_count: 2
				},
				// Hackity hack to get the term name. Appending an extra field otherwise
				// requires custom scripting which is not available.
				aggs: {
					names: {
						terms: {
							field: 'item.metadata.' + taxonomy + '.term.name',
							// Search term and match will be returned. Adding 'exclude'
							// slows the query x100
							size: 2
						}
					}
				}
			}
		});
	}, {});
}

function formatPostBody(options) {
	return {
		query: {
			bool: {
				must: [
					{
						match: formatSearchTerm(options)
					},
					{
						range: {
							'item.lifecycle.initialPublishDateTime': formatDateRange(options.range, 1)
						}
					}
				]
			}
		},
		sort: {
			'item.lifecycle.initialPublishDateTime': {
				order: 'desc'
			}
		},
		aggs: formatAggregations(options)
	};
}

function pickAndFlattenAggregations(data, options) {
	var aggregation = [];

	options.taxonomies.forEach(function(taxonomy) {
		if (!data.aggregations[taxonomy]) return;

		data.aggregations[taxonomy].buckets.forEach(function(tag) {
			var tagName1 = tag.names.buckets[0].key;
			var tagName2 = tag.names.buckets[1].key;

			aggregation.push({
				id: tag.key,
				taxonomy: taxonomy,
				doc_count: tag.doc_count,
				// Hackity hack. See `formatAggregations` above.
				name: tagName1 !== options.name ? tagName1 : tagName2
			});
		});
	});

	return aggregation;
}

function sortAggregation(aggregation) {
	return aggregation.sort(function(a, b){
		if (a.doc_count > b.doc_count) {
			return -1;
		}

		if (a.doc_count < b.doc_count) {
			return 1;
		}

		return 0;
	});
}

module.exports = function(options) {
	var postBody = formatPostBody(options);

	return eagerFetch('https://' + process.env.ELASTIC_SEARCH_HOST + '/v1_api_v2/item/_search?search_type=count', {
		body: JSON.stringify(postBody),
		method: 'POST',
		timeout: 3000
	})
		.then(function(response) {
			if (!response.ok) {
				logger.warn('Failed getting related tags for', options);
			}

			return response;
		})
		.then(fetchres.json)
		.then(function(data) {
			if (!data.aggregations) return [];

			return sortAggregation(pickAndFlattenAggregations(data, options));
		});
};

==> next-ft-api-client-3.7.0.tgz: package/src/related-tags.js <==
'use strict';

var fetchres = require('fetchres');
var logger = require('ft-next-logger');
var eagerFetch = require('./utils/eager-fetch');

function formatSearchTerm(options) {
	return Object.defineProperty({}, 'item.metadata.' + options.taxonomy + '.term.id', {
		value: options.id,
		enumerable: true
	});
}

function formatDateRange(from, to) {
	var dateFrom = new Date();
	var dateTo = new Date();

	dateFrom.setDate(dateFrom.getDate() - from);
	dateTo.setDate(dateTo.getDate() - to);

	return {
		gte: dateFrom.toISOString(),
		lt: dateTo.toISOString()
	};
}

function formatAggregations(options) {
	return options.taxonomies.reduce(function(previous, taxonomy) {
		return Object.defineProperty(previous, taxonomy, {
			enumerable: true,
			value: {
				terms: {
					field: 'item.metadata.' + taxonomy + '.term.id',
					exclude: options.id,
					min_doc_count: 2
				},
				// Hackity hack to get the term name. Appending an extra field otherwise
				// requires custom scripting which is not available.
				aggs: {
					names: {
						terms: {
							field: 'item.metadata.' + taxonomy + '.term.name',
							// Search term and match will be returned. Adding 'exclude'
							// slows the query x100
							size: 2
						}
					}
				}
			}
		});
	}, {});
}

function formatPostBody(options) {
	return {
		query: {
			bool: {
				must: [
					{
						match: formatSearchTerm(options)
					},
					{
						range: {
							'item.lifecycle.initialPublishDateTime': formatDateRange(options.range, 1)
						}
					}
				]
			}
		},
		sort: {
			'item.lifecycle.initialPublishDateTime': {
				order: 'desc'
			}
		},
		aggs: formatAggregations(options)
	};
}

function pickAndFlattenAggregations(data, options) {
	var aggregation = [];

	options.taxonomies.forEach(function(taxonomy) {
		if (!data.aggregations[taxonomy]) return;

		data.aggregations[taxonomy].buckets.forEach(function(tag) {
			var tagName1 = tag.names.buckets[0];
			var tagName2 = tag.names.buckets[1];

			aggregation.push({
				id: tag.key,
				taxonomy: taxonomy,
				doc_count: tag.doc_count,
				// Hackity hack. See `formatAggregations` above.
				name: tagName1.key === options.name && tagName2 ? tagName2.key : tagName1.key
			});
		});
	});

	return aggregation;
}

function sortAggregation(aggregation) {
	return aggregation.sort(function(a, b){
		if (a.doc_count > b.doc_count) {
			return -1;
		}

		if (a.doc_count < b.doc_count) {
			return 1;
		}

		return 0;
	});
}

module.exports = function(options) {
	var postBody = formatPostBody(options);

	return eagerFetch('https://' + process.env.ELASTIC_SEARCH_HOST + '/v1_api_v2/item/_search?search_type=count', {
		body: JSON.stringify(postBody),
		method: 'POST',
		timeout: 3000
	})
		.then(function(response) {
			if (!response.ok) {
				logger.warn('Failed getting related tags for', options);
			}

			return response;
		})
		.then(fetchres.json)
		.then(function(data) {
			if (!data.aggregations) return [];

			return sortAggregation(pickAndFlattenAggregations(data, options));
		});
};
